@@ -78,30 +78,26 @@ class RecordsDbDataSource(DataSource):
         connection_string = f"{dbtype}://{user}:{pw}@{host}{port}{service_name}"
         self.db = records.Database(connection_string, **kwargs)
 
-    def get_dataframe(self, arg_dict=None, buffer=False):
+    def get_dataframe(self, params=None, chunksize=None):
         """Get data as a pandas dataframe.
 
         Example::
 
             data_sources["my_datasource"].get_dataframe({"id": 387})
 
-        :param arg_dict: Query parameters to fill in query (e.g. `:id` with value 387)
-        :type arg_dict: optional dict
-        :param buffer: Currently not implemented
-        :type buffer: optional bool
+        :param params: Query parameters to fill in query (e.g. `:id` with value 387)
+        :type params: optional dict
+        :param chunksize: Currently not implemented
+        :type chunksize: optional bool
 
         :return: DataFrame object, possibly cached according to expires-config
         """
-        if buffer:
+        if chunksize:
             raise NotImplementedError("Buffered reading not supported yet")
             # the resulting `rows` of a query provides a nice way to do this, though
 
-        cached = self._try_get_cached_df()
-        if cached is not None:
-            return cached
-
         query = self.config["query"]
-        params = arg_dict or {}
+        params = params or {}
 
         logger.debug(
             "Fetching query {} with params {}...".format(
@@ -111,17 +107,15 @@ class RecordsDbDataSource(DataSource):
         rows = self.db.query(query, fetchall=True, **params)
         df = rows.export("df")
 
-        self._cache_df_if_required(df)
-
         return df
 
-    def get_raw(self, arg_dict=None, buffer=False):
+    def get_raw(self, params=None, chunksize=None):
         """Not implemented.
 
-        :param arg_dict: Query parameters to fill in query (e.g. `:id` with value 387)
-        :type arg_dict: optional dict
-        :param buffer: Currently not implemented
-        :type buffer: optional bool
+        :param params: Query parameters to fill in query (e.g. `:id` with value 387)
+        :type params: optional dict
+        :param chunksize: Currently not implemented
+        :type chunksize: optional bool
 
         :raises NotImplementedError:
 
