@@ -23,6 +23,12 @@ logger = logging.getLogger(__name__)
 # TODO: factor out datasource/model preparation code -- use model_actions instead of low-level functionality
 
 
+def get_api_base_url(config):
+    api_name = config["api"]["name"]
+    api_version = _get_major_api_version(config)
+    return "/{}/{}".format(api_name, api_version)
+
+
 def _get_major_api_version(config):
     match = re.match(r"\d+", config["model"]["version"])
     if match is None:
@@ -55,7 +61,8 @@ _type_lookup = {
     "number": float,
     "integer": int,
     "string": str,
-    "boolean": bool
+    "boolean": bool,
+    "array": list,  # https://medium.com/raml-api/array-enumeration-in-raml-b69950c75bb3
     # RAML Types: any, object, array, union via type expression,
     #             one of the following scalar types: number, boolean, string,
     #             date-only, time-only, datetime-only, datetime, file, integer, or nil
@@ -80,12 +87,21 @@ def _create_request_parser(resource_obj):
                 )
             )
 
+        # https://help.mulesoft.com/s/article/Repeat-query-parameters-using-RAML-1-0
+        try:
+            param_type, _ = p.type.split("[]")
+            is_array = True
+        except ValueError:
+            param_type = p.type
+            is_array = False
+        is_array = is_array or p.repeat
+
         parser.add_argument(
             p.name,
-            type=_type_lookup[p.type],
+            type=_type_lookup[param_type],
             required=p.required,
             default=p.default,
-            action="append" if p.repeat else "store",
+            action="append" if is_array else "store",
             choices=p.enum,
             help=str(p.description) + " - Error: {error_msg}",
         )
@@ -267,9 +283,7 @@ class ModelApi:
         logger.debug("Initializing RESTful API")
         api = Api(application)
 
-        api_name = config["api"]["name"]
-        api_version = _get_major_api_version(config)
-        api_url = "/{}/{}".format(api_name, api_version)
+        api_url = get_api_base_url(config)
 
         raml = _load_raml(config)
         res_normal, res_with_id, res_file = _get_resources(raml)
@@ -450,8 +464,7 @@ def generate_raml(
     param_hints = """# can be false if optional, then provide a default here or be prepared to deal with missing values in your prediction
         #default: {}  # only makes sense for required: false
         #minimum: 0  # optional, maximum, minLength and others are also possible."""
-    output = f"""
-#%RAML 0.8
+    output = f"""#%RAML 0.8
 ---
 title: Put title of your {api_name} API here
 baseUri: https://{{host}}/{api_name}/{{version}}
@@ -472,7 +485,7 @@ documentation:
         raml_type = _pd_type_lookup[type_str]
         example = repr(series[0])
         url_params += [quote_plus(col_name) + "=" + quote_plus(example)]
-        illegal_chars = ":.,[]'\"\\ \n\t"
+        illegal_chars = ':.,[]"\\ \n\t'
         quoted_col_name = (
             f"'{col_name}'"
             if any(c in illegal_chars for c in col_name)
@@ -491,7 +504,7 @@ documentation:
             example
         )
         if type_str == "category":
-            output += "\n" + str(list(series.cat.categories))
+            output += "\n        enum: " + str(list(series.cat.categories))
         param_hints = ""
 
     output += f"""
