@@ -104,6 +104,13 @@ def fill_nas(
         return df
 
 
+def ensure_dir_to(file_path):
+    path = os.path.dirname(file_path)
+    if path != "" and not os.path.exists(path):
+        logger.info("Creating missing path to file `{}`.".format(file_path))
+        os.makedirs(path)
+
+
 class SqlDataSource(DataSource):
     """DataSource for RedShift, Postgres, MySQL, SQLite, Oracle, Microsoft SQL (ODBC), and their dialects.
 
@@ -571,6 +578,7 @@ class FileDataSource(DataSource):
             expires: 0    # generic parameter, see documentation on DataSources
             tags: [train] # generic parameter, see documentation on DataSources and DataSinks
             options: {}   # used as **kwargs when fetching the data using `pandas.read_csv`
+            dtypes_path: ./some/file.dtypes # optional: location with the csv's column dtypes info
           my_raw_datasource:
             type: text_file  # raw files can also be of type `binary_file`
             path: /some/file.txt  # Can be URL
@@ -578,8 +586,15 @@ class FileDataSource(DataSource):
             tags: [train] # generic parameter, see documentation on DataSources and DataSinks
             options: {}   # used as **kwargs when fetching the data using `fh.read`
 
+    When loading `csv` or `euro_csv` type formats, you can use the setting `dtypes_path`
+    to specify a location with dtypes description for the csv (usually generated earlier
+    by using :class:`FileDataSink`'s `dtypes_path` setting). These dtypes will be enforced when
+    reading the csv, which helps avoid problems when `pandas.read_csv` interprets data differently than you do.
+    Use `dtypes_path` to enforce dtype parity between csv datasinks and datasources.
+
     Using the raw formats `binary_file` and `text_file`, you can read arbitrary data, as long as
-    it can be represented as a `bytes` or a `str` object, respectively. Please note that while possible, it is not
+    it can be represented as a `bytes` or a `str` object, respectively. `text_file` uses UTF-8
+    encoding. Please note that while possible, it is not
     recommended to persist `DataFrame`s this way, because by adding format-specific code to your
     model, you're giving up your code's independence from the type of `DataSource`/`DataSink`.
     Here's an example for unpickling an arbitrary object::
@@ -617,6 +632,7 @@ class FileDataSource(DataSource):
 
         self.type = ds_type
         self.path = datasource_config["path"]
+        self.dtypes_path = datasource_config.get("dtypes_path")
 
     def get_dataframe(
         self, params: Dict = None, chunksize: Optional[int] = None
@@ -640,10 +656,23 @@ class FileDataSource(DataSource):
         kw_options = self.options
 
         logger.debug(
-            "Loading type {} file {} with chunksize {} and options {}...".format(
-                self.type, self.path, chunksize, kw_options
+            "Loading type {} file {} with dtypes_path {}, chunksize {} and options {}...".format(
+                self.type, self.path, self.dtypes_path, chunksize, kw_options
             )
         )
+        if self.dtypes_path is not None:
+            input_dtypes = pd.read_csv(self.dtypes_path).set_index("columns")
+            kw_options["dtype"] = {
+                item: input_dtypes.loc[item]["dtypes"]
+                for item in input_dtypes.index
+                if input_dtypes.loc[item]["dtypes"] != "datetime"
+            }
+            kw_options["parse_dates"] = [
+                item
+                for item in input_dtypes.index
+                if input_dtypes.loc[item]["dtypes"] == "datetime"
+            ]
+
         if self.type == "csv":
             df = pd.read_csv(self.path, chunksize=chunksize, **kw_options)
         elif self.type == "euro_csv":
@@ -693,7 +722,7 @@ class FileDataSource(DataSource):
 
         raw: Raw
         if self.type == "text_file":
-            with open(self.path, "r") as txt_file:
+            with open(self.path, "r", encoding="utf-8") as txt_file:
                 raw = txt_file.read(**kw_options)
         elif self.type == "binary_file":
             with open(self.path, "rb") as bin_file:
@@ -721,14 +750,22 @@ class FileDataSink(DataSink):
             path: /some/file.csv  # Can be URL, uses `df.to_csv` internally
             tags: [train] # generic parameter, see documentation on DataSources and DataSinks
             options: {}   # used as **kwargs when fetching the data using `df.to_csv`
+            dtypes_path: ./some/file.dtypes # optional: location for saving the csv's column dtypes info
           my_raw_datasink:
             type: text_file  # raw files can also be of type `binary_file`
             path: /some/file.txt  # Can be URL
             tags: [train] # generic parameter, see documentation on DataSources and DataSinks
             options: {}   # used as **kwargs when writing the data using `fh.write`
 
+    When saving `csv` or `euro_csv` type formats, you can use the setting `dtypes_path`
+    to specify a location where to save dtypes descriptions for the csv (that you can use later
+    with :class:`FileDataSource`'s `dtypes_path` setting). These dtypes will be enforced when
+    reading the csv, which helps avoid problems when `pandas.read_csv` interprets data differently than you do.
+    Use `dtypes_path` to enforce dtype parity between csv datasinks and datasources.
+
     Using the raw formats `binary_file` and `text_file`, you can persist arbitrary data, as long as
-    it can be represented as a `bytes` or a `str` object, respectively. Please note that while possible, it is not
+    it can be represented as a `bytes` or a `str` object, respectively. `text_file` uses UTF-8
+    encoding. Please note that while possible, it is not
     recommended to persist `DataFrame`s this way, because by adding format-specific code to your
     model, you're giving up your code's independence from the type of `DataSource`/`DataSink`.
     Here's an example for pickling an arbitrary object::
@@ -765,6 +802,7 @@ class FileDataSink(DataSink):
 
         self.type = ds_type
         self.path = datasink_config["path"]
+        self.dtypes_path = datasink_config.get("dtypes_path")
 
     def put_dataframe(
         self,
@@ -772,9 +810,10 @@ class FileDataSink(DataSink):
         params: Dict = None,
         chunksize: Optional[int] = None,
     ) -> None:
-        """Write a pandas dataframe to file.
+        """Write a pandas dataframe to file and optionally the dtypes if included in the configuration.
         The default is not to save the dataframe's row index.
         Configure the DataSink's `options` dict to pass keyword arguments to `my_df.to_csv`.
+        If the directory path leading to the file does not exist, it will be created.
 
         Example::
 
@@ -797,13 +836,28 @@ class FileDataSink(DataSink):
             kw_options["index"] = False
 
         logger.debug(
-            "Writing dataframe to type {} file {} with options {}...".format(
-                self.type, self.path, kw_options
+            "Writing dataframe to type {} file {} with options {} and dtypes_path {}...".format(
+                self.type, self.path, kw_options, self.dtypes_path
             )
         )
+        if self.dtypes_path:
+            dtypes_file = pd.DataFrame(
+                dataframe.dtypes, columns=["dtypes"]
+            ).rename_axis("columns")
+            dtypes_file.loc[
+                dtypes_file["dtypes"] == "object", "dtypes"
+            ] = "str"
+            dtypes_file.loc[
+                dtypes_file["dtypes"] == "datetime64[ns]", "dtypes"
+            ] = "datetime"
+            ensure_dir_to(self.dtypes_path)
+            dtypes_file.to_csv(self.dtypes_path)
+
         if self.type == "csv":
+            ensure_dir_to(self.path)
             dataframe.to_csv(self.path, **kw_options)
         elif self.type == "euro_csv":
+            ensure_dir_to(self.path)
             dataframe.to_csv(self.path, sep=";", decimal=",", **kw_options)
         else:
             raise TypeError(
@@ -817,6 +871,7 @@ class FileDataSink(DataSink):
         chunksize: Optional[int] = None,
     ) -> None:
         """Write raw (unstructured) data to file.
+        If the directory path leading to the file does not exist, it will be created.
 
         Example::
 
@@ -842,10 +897,14 @@ class FileDataSink(DataSink):
             )
         )
         if self.type == "text_file":
+            if "encoding" not in kw_options:
+                kw_options["encoding"] = "utf-8"
+            ensure_dir_to(self.path)
             with open(self.path, "w", **kw_options) as txt_file:
                 raw_str: str = cast(str, raw_data)
                 txt_file.write(raw_str)
         elif self.type == "binary_file":
+            ensure_dir_to(self.path)
             with open(self.path, "wb", **kw_options) as bin_file:
                 raw_bytes: bytes = cast(bytes, raw_data)
                 bin_file.write(raw_bytes)
